#if defined(__cplusplus)
extern "C"
{
#endif

#include "log.h"
#include "format.h"
#include "bp.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#if defined(__cplusplus)
}
#endif



#ifndef __CINT__

//--- ROOT includes ---

#include <TROOT.h>
#include <TMath.h>
#include <TRandom.h>

TROOT root("Rint","The ROOT Interactive Interface");

#endif



int main (int argc, char *argv[])
{
  Array breakPoints;
  BreakPoint *currBP;
  BreakPointRead *currBPR;
  int i,j;
  int minNumReads,minNumUniqueOffsets,minNumReadsForKS,numPossibleOffsets;
  double pValueCutoffForKS;
  Array offsets;
  Array randomNumbers;
  double *observedOffsets;
  double *randomOffsets;

  if (argc != 6) {
    usage ("%s <minNumReads> <minNumUniqueOffsets> <minNumReadsForKS> <pValueCutoffForKS> <numPossibleOffsets>",argv[0]);
  }
  minNumReads = atoi (argv[1]);
  minNumUniqueOffsets = atoi (argv[2]);
  minNumReadsForKS = atoi (argv[3]);
  pValueCutoffForKS = atof (argv[4]);
  numPossibleOffsets = atoi (argv[5]);
  bp_init ("-");
  offsets = arrayCreate (100,int);
  randomNumbers = arrayCreate (100,int);
  breakPoints = bp_getBreakPoints ();
  for (i = 0; i < arrayMax (breakPoints); i++) {
    currBP = arrp (breakPoints,i,BreakPoint);
    arrayClear (offsets);
    for (j = 0; j < arrayMax (currBP->breakPointReads); j++) {
      currBPR = arrp (currBP->breakPointReads,j,BreakPointRead);
      array (offsets,arrayMax (offsets),int) = currBPR->offset;
    }
    arraySort (offsets,(ARRAYORDERF)arrayIntcmp);
    arrayUniq (offsets,NULL,(ARRAYORDERF)arrayIntcmp);
    if (arrayMax (currBP->breakPointReads) >= minNumReads && arrayMax (currBP->breakPointReads) < minNumReadsForKS) {        
      if (arrayMax (offsets) >= minNumUniqueOffsets) {
        puts (bp_writeBreakPoint (currBP));
      }
    }
    else if (arrayMax (currBP->breakPointReads) >= minNumReads && arrayMax (currBP->breakPointReads) >= minNumReadsForKS) {
      arrayClear (randomNumbers);
      for (j = 0; j < arrayMax (offsets); j++) {
        array (randomNumbers,arrayMax (randomNumbers),int) = rand () % numPossibleOffsets;
      }
      arraySort (randomNumbers,(ARRAYORDERF)arrayIntcmp);
      observedOffsets = (double*)hlr_malloc (arrayMax (offsets) * sizeof (double)); 
      randomOffsets = (double*)hlr_malloc (arrayMax (offsets) * sizeof (double)); 
      for (j = 0; j < arrayMax (offsets); j++) {
        observedOffsets[j] = arru (offsets,j,int);
        randomOffsets[j] = arru (randomNumbers,j,int);
      }
      if (pValueCutoffForKS < TMath::KolmogorovTest(arrayMax (offsets),observedOffsets,arrayMax (offsets),randomOffsets,"")) {
        puts (bp_writeBreakPoint (currBP));
      }
      hlr_free (observedOffsets);
      hlr_free (randomOffsets);
    }
  }
  bp_deInit ();
  return 0;
}

