#include "log.h"
#include "format.h"
#include "linestream.h"
#include "mrf.h"
#include <stdio.h>



static void freeMemory (char *read, char *quality, char *chromosome, char *partnerChromosome, Texta tokens) 
{
  hlr_free (read);
  hlr_free (quality);
  hlr_free (chromosome);
  hlr_free (partnerChromosome);
  textDestroy (tokens);
}



static char convertStrand (char strand) 
{
  if (strand == 'F') {
    return '+';
  }
  if (strand != 'R') {
    die ("Unexpected strand: %c",strand);
  }
  return '-';
}

int isJunction( char* chromosome ) {
  if( strstr( chromosome, "|") != NULL )
    return 1;
  else
    return 0;
}

int main (int argc, char *argv[])
{
  LineStream ls1,ls2;
  char *line1,*line2;
  WordIter w;
  char *read;
  char *quality;
  int readLength;
  int position;
  int offset;
  char *chromosome=NULL;
  char *partnerChromosome=NULL;
  int chrEnd1, startEnd1;
  int chrEnd2, startEnd2;
  char strand;
  char *contig;
  char *partnerContig;
  char partnerStrand;
  char filter;
  char *pos;
  Texta tokens;
  Stringa id;
  FILE *fp;
  Stringa buffer;
  int countExport = 0;
  int countMRF = 0;
  int junction1, junction2;

  if (argc != 4) {
    usage ("%s <prefix> <exportFile1> <exportFile2>",argv[0]);
  }
  id = stringCreate (100);
  buffer = stringCreate (100);
  stringPrintf (buffer,"%s_allReads.txt",argv[1]);
  fp = fopen (string (buffer),"w");
  if (fp == NULL) {
    die ("Unable to open file: %s",string (buffer));
  }

  printf ("%s\t%s\n",MRF_COLUMN_NAME_BLOCKS,MRF_COLUMN_NAME_SEQUENCE);
  if( strEndsWith( argv[2], ".gz" ) ) { // if export files are zipped, it uses zcat and createFromPipe
    stringPrintf( buffer, "zcat %s", argv[2] );
    ls1 = ls_createFromPipe ( string(buffer) );
    stringPrintf( buffer, "zcat %s", argv[3] );
    ls2 = ls_createFromPipe ( string(buffer) );
  } else {
    ls1 = ls_createFromFile (argv[2]);
    ls2 = ls_createFromFile (argv[3]);
  }
  while ((line1 = ls_nextLine (ls1)) && (line2 = ls_nextLine (ls2))) {
    //warn("%s\n%s", line1, line2);
    countExport++;
    tokens = textFieldtok (line2,"\t");
    stringClear (id);
    w = wordIterCreate (line1,"\t",0);
    stringAppendf (id,"%s_",wordNext (w)); // 1 machine
    wordNext (w); // 2 run number
    stringAppendf (id,"%s_",wordNext (w)); // 3 lane
    stringAppendf (id,"%s_",wordNext (w)); // 4 tile
    stringAppendf (id,"%s_",wordNext (w)); // 5 x coordinate of cluster
    stringAppendf (id,"%s_",wordNext (w)); // 6 y coordinate of cluster
    stringAppendf (id,"%s",wordNext (w)); // 7 index string
    wordNext (w); // 8 read number
    read = hlr_strdup (wordNext (w)); // 9 read
    readLength = strlen (read); 
    quality = hlr_strdup (wordNext (w)); // 10 quality
    chromosome = hlr_strdup (wordNext (w)); // 11 match chromosome
    contig = wordNext (w); // 12 match contig, true if splice junction
    position = atoi (wordNext (w)); // 13 match position
    strand = wordNext (w)[0]; // 14 match strand
    wordNext (w); // 15 match descriptor
    wordNext (w); // 16 single read alignment score
    wordNext (w); // 17 paired end alignment score
    partnerChromosome = hlr_strdup (wordNext (w)); // 18 partner chromosome
    partnerContig = wordNext (w); // 19 partner contig
    offset = atoi (wordNext (w)); // 20 partner offset
    partnerStrand = wordNext (w)[0]; // 21 partner strand
    filter = wordNext (w)[0]; // 22 filter
    wordIterDestroy (w);
    junction1=0; 
    junction2=0;
    if(filter != 'Y') {
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
      continue;
    }
    // check for NM reads from exportFile1
    if (strEqual (chromosome,"NM")) {
      fprintf (fp,"%s\n",read);
    }
    // check for NM reads from exportFile2
    if (strEqual (textItem (tokens,10),"NM")) {
      fprintf (fp,"%s\n",textItem (tokens,8));
    }
    // skip reads mapping to the mitochondrial chromosome
    if (strchr (chromosome,'M') || strchr (partnerChromosome,'M')) {
      fprintf (fp,"%s\n",read);
      fprintf (fp,"%s\n",textItem (tokens,8));
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
      continue;
    }    
    // skip reads with QC or multi-hits
    if (strStartsWith (chromosome,"QC") || strStartsWith (partnerChromosome,"QC") || strchr( chromosome, ':') ) {
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
      continue;
    }    
    if( strStartsWith( chromosome, "ribosomal") || strStartsWith( partnerChromosome, "ribosomal")  ) {
      fprintf (fp,"%s\n",read);
      fprintf (fp,"%s\n",textItem (tokens,8));
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 	  
      continue;
    }
    
    if( strchr( chromosome, ':' ) || strchr( partnerChromosome, ':' ) ) { // multi-hits
      fprintf (fp,"%s\n",read);
      fprintf (fp,"%s\n",textItem (tokens,8));
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 	  
      continue;
    }

    Stringa end1 = stringCreate( 30 );
    Stringa end2 = stringCreate( 30 );
    
    if( strStartsWith( chromosome, "chr") ) { // genomic
      pos = strchr (chromosome,'.');	  
      *pos = '\0';
      chrEnd1 = atoi( chromosome+3 ); 
      startEnd1 = position;
      if( position<0 ) 
	continue;
      stringPrintf( end1, "%s:%c:%d:%d:1:%d", chromosome, convertStrand (strand), position, position + readLength -1, readLength );
    } else if( strstr( chromosome, "junction" ) ) { // junction
      Texta location = textFieldtokP ( contig, "|" );
      int tilestart1 = atoi( textItem( location, 1) );
      int tilestart2 = atoi( textItem( location, 2) );
      int tileSize = atoi( textItem( location, 3 ) );
      if( (tileSize - position + 1 ) >= readLength  || ( (tileSize - position) < 0) ) 
	continue;
      int targetStart1 = tilestart1 + position;
      int targetEnd1   = tilestart1 + tileSize;
      int queryStart1  = 1;
      int queryEnd1    = (targetEnd1 - targetStart1 + 1);
      int basesOnSecondBlock = readLength - (queryEnd1 - queryStart1 + 1);

      int targetStart2 = tilestart2 + 1 ; // to check why + 1 -- it seems the coordinate for the second exon is 0 based
      int targetEnd2   = tilestart2 + 1 + basesOnSecondBlock - 1;
      int queryStart2  = queryEnd1 + 1;
      int queryEnd2    = readLength;
      stringPrintf( end1, "%s:%c:%d:%d:%d:%d,%s:%c:%d:%d:%d:%d", 
		    textItem( location, 0),convertStrand (strand),targetStart1,targetEnd1, queryStart1, queryEnd1, // first block
		    textItem( location, 0),convertStrand (strand),targetStart2,targetEnd2, queryStart2, queryEnd2  // second block
		    );
      chrEnd1 = atoi( textItem( location, 0)+3  ); 
      startEnd1 = targetStart1;
      textDestroy( location );
      junction1 = 1;
    } else {
      warn( "End 1 not in a proper format:%s\tchr:%s\tcontig:%s\tposition:%d\tpartnStrand:%c", read, chromosome, contig, position, strand);
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
      continue;
    }

    if( strStartsWith( partnerChromosome, "chr") ) { // genomic - inter
      pos = strchr (partnerChromosome,'.');	  
      *pos = '\0';
      chrEnd2 = atoi( partnerChromosome+3 );
      startEnd2 = offset;
      stringPrintf( end2, "%s:%c:%d:%d:1:%d", partnerChromosome, convertStrand (partnerStrand), offset, offset + readLength - 1, readLength );
    } else if( partnerChromosome[0] == '\0' && strStartsWith( chromosome, "chr") && ( partnerStrand=='F' || partnerStrand=='R' ) )  {      // genomic - intra
      pos = strchr (chromosome,'.');	  
      if( pos != NULL ) *pos = '\0';
      stringPrintf( end2, "%s:%c:%d:%d:1:%d", chromosome, convertStrand (partnerStrand), position + offset, position + offset + readLength - 1, readLength );
      chrEnd2=atoi( chromosome+3); 
      startEnd2 = position + offset;
    } else if( strstr( partnerChromosome, "junction" ) ) { // junction
      Texta location = textFieldtokP ( partnerContig, "|" );
      //strReplace( &partnerChromosome,  textItem( location, 0));
      int tilestart1 = atoi( textItem( location, 1) );
      int tilestart2 = atoi( textItem( location, 2) );
      int tileSize = atoi( textItem( location, 3 ) );
      if( (tileSize-offset + 1) >= readLength || ( (tileSize - offset) < 0) )
	continue;
      int targetStart1 = tilestart1 + offset;
      int targetEnd1   = tilestart1 + tileSize;
      int queryStart1  = 1;
      int queryEnd1    = (targetEnd1 - targetStart1 + 1);
      int basesOnSecondBlock = readLength - (queryEnd1 - queryStart1 + 1);

      int targetStart2 = tilestart2 + 1 ; // to check why + 1 -- it seems the coordinate for the second exon is 0 based
      int targetEnd2   = tilestart2 + 1 + basesOnSecondBlock - 1;
      int queryStart2  = queryEnd1 + 1;
      int queryEnd2    = readLength;
      stringPrintf( end2, "%s:%c:%d:%d:%d:%d,%s:%c:%d:%d:%d:%d", 
		    textItem( location, 0),convertStrand (strand),targetStart1,targetEnd1, queryStart1, queryEnd1, // first block
		    textItem( location, 0),convertStrand (strand),targetStart2,targetEnd2, queryStart2, queryEnd2  // second block
		    );
      chrEnd2=atoi(textItem( location, 0)+3 );
      startEnd2 = targetStart1;
      textDestroy( location );
      junction2=1;
    } else if ( partnerStrand=='N' && offset == 0 ){ // no offset 
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
      continue;
    } else { // probably overlapping junction/splice junction
      warn( "**\nOverlapping reads on a junction? Prognosys splice junction?\nreadEnd1:%s\treadEnd2:%s\tchr:%s\tpartnChr:%s\tContig:%s\tPosition:%d\tpartnContig:%s\tpartnPosition:%d\tpartnStrand:%c\n", read, textItem( tokens,8 ), chromosome, partnerChromosome, contig, position, partnerContig, offset, partnerStrand);
      freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
      continue;
    }
    
    if( chrEnd1 < chrEnd2   ) {
      printf ("%s|%s\t%s|%s\n", string(end1), string(end2), read, textItem( tokens,8 ));
    } else {
      if( chrEnd1 > chrEnd2 ) {
	printf ("%s|%s\t%s|%s\n", string(end2), string(end1), textItem( tokens,8 ), read);
      } else {
	if ((strand == 'R' && partnerStrand == 'F') || (strand == 'F' && partnerStrand == 'R') && 
	    ( junction1 == 0 && junction2 == 0) ) {	  	  	     
	  if( (startEnd2-startEnd1) > 0 ) {
	    printf ("%s|%s\t%s|%s\n", string(end1), string(end2), read, textItem( tokens,8 ));
	  } else {
	    printf ("%s|%s\t%s|%s\n", string(end2), string(end1), textItem( tokens,8 ), read);
	  }
	} else {
	  if( junction1 == 1 || junction2 == 1 ) {
	    if( (startEnd2-startEnd1) > 0 ) {
	      printf ("%s|%s\t%s|%s\n", string(end1), string(end2), read, textItem( tokens,8 ));
	    } else {
	      printf ("%s|%s\t%s|%s\n", string(end2), string(end1), textItem( tokens,8 ), read);
	    }
	  } else {
	    freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
	    continue;
	  }
	}    	      	  
      }	
    }
    countMRF++;
    freeMemory (read,quality,chromosome,partnerChromosome,tokens); 
  }
  ls_destroy (ls1);
  ls_destroy (ls2);
  fclose (fp);
  
  stringPrintf (buffer,"%s.meta",argv[1]);
  fp = fopen (string (buffer),"w");
  if (fp == NULL) {
    die ("Unable to open file: %s",string (buffer));
  } 
  fprintf(fp, "Total_number_reads\t%d\nMapped_reads\t%d", countExport,countMRF);
  fclose (fp);
  stringDestroy (buffer);
  stringDestroy (id);
  return 0;
}



