#include "log.h"
#include "format.h"
#include "bowtieParser.h"
#include "bp.h"
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include "geneFusionsConfig.h"



int main (int argc, char *argv[])
{
  Array breakPoints;
  BreakPoint *currBP;
  BreakPointRead *currBPR;
  int i,j;
  Stringa buffer,cmd;
  FILE *fp;
  BowtieQuery *currBQ;
  Texta invalidJunctions;
  int index;
  char *str = NULL;

  buffer = stringCreate (100);
  stringPrintf (buffer,"bpJunctionReads_%d.fa",getpid ());
  fp = fopen (string (buffer),"w");
  if (fp == NULL) {
    die ("Unable to open file: %s",string (buffer));
  }
  bp_init ("-");
  breakPoints = bp_getBreakPoints ();
  for (i = 0; i < arrayMax (breakPoints); i++) {
    currBP = arrp (breakPoints,i,BreakPoint);
    for (j = 0; j < arrayMax (currBP->breakPointReads); j++) {
      currBPR = arrp (currBP->breakPointReads,j,BreakPointRead);
      fprintf (fp,">%s|%s\n%s\n",currBP->tileCoordinate1,currBP->tileCoordinate2,currBPR->read);
    }
  }
  bp_deInit ();
  fclose (fp);

  invalidJunctions = textCreate (1000);
  cmd = stringCreate (100);
  stringPrintf (cmd,"bowtie --quiet -p 4 -n 0 -f %s/%s/%s %s",BOWTIE_INDEXES,BOWTIE_GENOME, BOWTIE_GENOME, string (buffer));
  bowtieParser_initFromPipe (string (cmd));
  while (currBQ = bowtieParser_nextQuery ()) {
    textAdd (invalidJunctions,currBQ->sequenceName);
  }
  bowtieParser_deInit ();
  stringPrintf (cmd,"rm -f %s",string (buffer));
  hlr_system (string (cmd),0);

  arraySort (invalidJunctions,(ARRAYORDERF)arrayStrcmp);
  arrayUniq (invalidJunctions,NULL,(ARRAYORDERF)arrayStrcmp);
  for (i = 0; i < arrayMax (breakPoints); i++) {
    currBP = arrp (breakPoints,i,BreakPoint);
    stringPrintf (buffer,"%s|%s",currBP->tileCoordinate1,currBP->tileCoordinate2);
    strReplace (&str,string (buffer));
    if (!arrayFind (invalidJunctions,&str,&index,(ARRAYORDERF)arrayStrcmp)) {
      puts (bp_writeBreakPoint (currBP));
    }
  }
  stringDestroy (buffer);
  stringDestroy (cmd);
  return 0;
}
